@@ -16,8 +16,8 @@
 //!
 //! Example
 //! ```no_run
-//! use ssip_client::{FifoBuilder, ClientName};
-//! let mut client = FifoBuilder::new().build()?;
+//! use ssip_client::{fifo, ClientName};
+//! let mut client = fifo::Builder::new().build()?;
 //! client
 //!     .set_client_name(ClientName::new("joe", "hello"))?
 //!     .check_client_name_set()?;
@@ -30,12 +30,21 @@
 mod protocol;
 
 mod client;
-mod constants;
-mod fifo;
 mod types;
 
-pub use client::{Client, ClientError, ClientName, ClientResult, ClientStatus};
+pub mod constants;
+pub mod fifo;
+
+#[cfg(not(feature = "async-mio"))]
+pub use client::Client;
+
+pub use client::{ClientError, ClientName, ClientResult, ClientStatus};
 pub use constants::*;
-pub use fifo::FifoBuilder;
 pub use types::StatusLine;
 pub use types::*;
+
+#[cfg(feature = "async-mio")]
+mod async_mio;
+
+#[cfg(feature = "async-mio")]
+pub use async_mio::{AsyncClient, Request, Response};
